from typing import Optional, Tuple

from thestage_core.config import THESTAGE_API_URL
from thestage_core.exceptions.auth_exception import AuthException
from thestage_core.entities.config_entity import ConfigEntity
from thestage_core.services.clients.thestage_api.api_client import TheStageApiClientCore
from thestage_core.services.config_provider.config_provider import ConfigProviderCore
from thestage_core.services.validation_service import ValidationServiceCore


def global_check_validation() -> Optional[ConfigEntity]:
    config_provider = ConfigProviderCore(
        auto_create=True,
        only_global=True,
    )
    config: ConfigEntity = config_provider.get_full_config(only_global=True)

    validation_service = ValidationServiceCore(
        thestage_api_client=TheStageApiClientCore(url=THESTAGE_API_URL),
        config_provider=config_provider,
    )

    if validation_service.is_present_token(config):
        if validation_service.validate_token(config.main.auth_token):
            return config
        else:
            raise AuthException("Your token is not valid, please update him, you can not use QLIP")
    else:
        raise AuthException("Not found TheStage API token, please use thestage cli to initialize auth config")



from enum import Enum
from typing import List


class ContainerStatusEnumDto(str, Enum):
    FAILED: str = 'FAILED'
    BUSY: str = 'BUSY'
    DEAD: str = 'DEAD'
    CREATING: str = 'CREATING'
    CREATING_FAILED: str = 'CREATING_FAILED'
    STARTING: str = 'STARTING'
    RUNNING: str = 'RUNNING'
    STOPPING: str = 'STOPPING'
    STOPPED: str = 'STOPPED'
    RESTARTING: str = 'RESTARTING'
    DELETING: str = 'DELETING'
    DELETED: str = 'DELETED'
    UNKNOWN: str = 'UNKNOWN'
    ALL: str = 'ALL'

    @staticmethod
    def find_special_status(statuses: List['ContainerStatusEnumDto']) -> bool:
        q = list(filter(lambda x: True if x == ContainerStatusEnumDto.ALL else False, statuses))
        return True if q else False



from enum import Enum


class ProviderNameEnumDto(str, Enum):
    Amazon: str = 'Amazon'
    DigitalOcean: str = 'DigitalOcean'
    Exoscale: str = 'Exoscale'
    Vultr: str = 'Vultr'
    UNKNOWN: str = 'UNKNOWN'



from typing import Optional

from pydantic import BaseModel, Field

from thestage_core.entities.enums.repo_type import RepoType
from thestage_core.entities.enums.config_type import ConfigType
from thestage_core.services.clients.thestage_api.dtos.user_profile import UserProfileResponse


class MainConfigEntity(BaseModel):
    auth_token: Optional[str] = Field(None, alias='thestage_auth_token')
    config_local_path: Optional[str] = Field(None, alias='thestage_config_local_path')
    config_global_path: Optional[str] = Field(None, alias='thestage_config_global_path')
    config_file_name: Optional[str] = Field(None, alias='thestage_config_file_name')
    config_local_dir: Optional[str] = Field(None, alias='thestage_config_local_dir')
    config_api_link: Optional[str] = Field(None, alias='thestage_api_url')


class DaemonConfigEntity(BaseModel):
    daemon_token: Optional[str] = Field(None, alias='daemon_token')
    backend_api_url: Optional[str] = Field(None, alias='backend_api_url')


class RuntimeConfigEntity(BaseModel):
    working_directory: Optional[str] = Field(None, alias='working_directory')
    profile: Optional[UserProfileResponse] = Field(None, alias='profile')
    config_type: ConfigType = Field(None, alias='config_type')
    repo_url: Optional[str] = Field(None, alias='thestage_repo_url')
    repo_type: Optional[RepoType] = Field(RepoType.UNKNOWN, alias='thestage_repo_type')


class ConfigEntity(BaseModel):
    main: MainConfigEntity = Field(default_factory=MainConfigEntity, alias='main')
    runtime: RuntimeConfigEntity = Field(default_factory=RuntimeConfigEntity, alias="runtime")
    daemon: DaemonConfigEntity = Field(default_factory=DaemonConfigEntity, alias="daemon")
    start_on_daemon: bool = Field(False, alias='start_on_daemon')



import configparser
import json
import os
from json import JSONDecodeError
from pathlib import Path
from typing import Optional, Dict, Any

import dotenv

from thestage_core.entities.enums.repo_type import RepoType

from thestage_core.config import THESTAGE_CONFIG_DIR, THESTAGE_CONFIG_FILE, THESTAGE_AUTH_TOKEN, THESTAGE_API_URL, \
    THESTAGE_DAEMON_ENV_PATH
from thestage_core.entities.enums.config_type import ConfigType
from thestage_core.services.filesystem_service import FileSystemServiceCore
from thestage_core.entities.config_entity import ConfigEntity, MainConfigEntity
from thestage_core.exceptions.file_system_exception import FileSystemException


class ConfigProviderCore:

    _current_path: Optional[Path] = None
    _local_config_path: Optional[Path] = None
    _local_config_file: Optional[Path] = None
    _global_config_path: Optional[Path] = None
    _global_config_file: Optional[Path] = None
    _config_parser: configparser.ConfigParser
    _file_system_service = FileSystemServiceCore
    _auto_create: bool = True

    def __init__(
            self,
            project_path: Optional[str] = None,
            auto_create: bool = True,
            only_global: bool = False,
    ):
        self._file_system_service = FileSystemServiceCore()
        self._auto_create = auto_create
        if not only_global:
            self._current_path = self._file_system_service.get_path(project_path, self._auto_create)

        self._config_parser = configparser.ConfigParser()
        self._get_config_paths(only_global=only_global)
        self.init_config_files(auto_create=self._auto_create, only_global=only_global)

    def _get_config_paths(self, only_global: bool = False):
        folder_name = self.__get_config_folder_name()
        if not only_global:
            self._local_config_path = self._current_path.joinpath(folder_name)
        else:
            self._local_config_path = None

        home_dir = self._file_system_service.get_home_path()
        self._global_config_path = home_dir.joinpath(folder_name)

    def __get_config_folder_name(self,) -> Path:
        return self._file_system_service.get_path(f"{THESTAGE_CONFIG_DIR}", False)

    def __get_config_file_name(self,) -> Path:
        return self._file_system_service.get_path(f"{THESTAGE_CONFIG_FILE}", False)

    def init_config_files(self, auto_create: bool = True, only_global: bool = False) -> None:
        # create local config
        if not only_global:
            if self._local_config_path and auto_create:
                self._file_system_service.create_if_not_exists(self._local_config_path)

            self._local_config_file = self._local_config_path.joinpath(self.__get_config_file_name())
            if auto_create and (not self._local_config_file or not self._local_config_file.exists()):
                self._file_system_service.create_if_not_exists_file(self._local_config_file)

        # create global config
        if self._global_config_path:
            if not self._global_config_path.exists():
                self._file_system_service.create_if_not_exists(self._global_config_path)

        self._global_config_file = self._global_config_path.joinpath(self.__get_config_file_name())
        if auto_create and (not self._global_config_file or not self._global_config_file.exists()):
            self._file_system_service.create_if_not_exists_file(self._global_config_file)

    @staticmethod
    def _read_data_from_env() -> Optional[Dict[str, Any]]:
        result = {}
        tsr_auth_token = THESTAGE_AUTH_TOKEN
        tsr_config_file_name = THESTAGE_CONFIG_FILE
        tsr_config_local_dir = THESTAGE_CONFIG_DIR
        tsr_config_api_url = THESTAGE_API_URL

        if tsr_auth_token or tsr_config_file_name or tsr_config_local_dir:
            result = {}
            result['main'] = {}
            if tsr_auth_token:
                result['main']['thestage_auth_token'] = tsr_auth_token
            if tsr_config_file_name:
                result['main']['thestage_config_file_name'] = tsr_config_file_name
            if tsr_config_local_dir:
                result['main']['thestage_config_local_dir'] = tsr_config_local_dir
            if tsr_config_api_url:
                result['main']['thestage_api_url'] = tsr_config_api_url

        return result

    def __read_config_file(self, path: Path) -> Dict[str, Any]:
        result = {}
        try:
            if path and path.exists():
                with path.open("r") as file:
                    try:
                        if os.stat(path).st_size != 0:
                            result = json.load(file)
                    except JSONDecodeError:
                        pass
        except OSError:
            raise FileSystemException("Error open local config file")
        return result

    def _read_full_local_config(self,) -> Dict[str, Any]:
        return self.__read_config_file(self._local_config_file)

    def _read_full_global_config(self,) -> Dict[str, Any]:
        return self.__read_config_file(self._global_config_file)

    @staticmethod
    def __update_values_dict(src: Dict, new_values: Dict):
        if 'main' in new_values:
            if 'main' in src:
                src['main'].update(new_values['main'])
            else:
                src['main'] = new_values['main']

    def get_full_config(self, only_global: bool = False, check_daemon: bool = False,) -> ConfigEntity:
        values = {}

        # for now only global
        config_type: ConfigType = ConfigType.GLOBAL

        # read global data
        data = self._read_full_global_config()
        if data:
            self.__update_values_dict(src=values, new_values=data)
            config_type: ConfigType = ConfigType.GLOBAL

        if not only_global:
            # read local data
            data = self._read_full_local_config()
            if data:
                self.__update_values_dict(src=values, new_values=data)
                config_type: ConfigType = ConfigType.LOCAL

        # read end data
        data = self._read_data_from_env()

        if data:
            self.__update_values_dict(src=values, new_values=data)
        #if data:
        #    config_type: ConfigType = ConfigType.ENV

        config = ConfigEntity.model_validate(values)

        if self._current_path:
            config.runtime.working_directory = str(self._current_path)
        config.runtime.config_type = config_type
        if self._local_config_path and not config.main.config_local_path:
            config.main.config_local_path = str(self._local_config_path)
        if self._global_config_path and not config.main.config_global_path:
            config.main.config_global_path = str(self._global_config_path)

        if check_daemon:
            self.check_for_daemon(config=config)

        return config

    def create_runtime_config(
            self,
            token: Optional[str] = None,
            repo_url: Optional[str] = None,
            no_dialog: bool = False,
    ) -> ConfigEntity:

        config = self.get_full_config()
        if not (config.main and config.main.auth_token):
            config.main.auth_token = token

        if self._current_path:
            config.runtime.working_directory = str(self._current_path)

        if not config.runtime.config_type:
            config.runtime.config_type = ConfigType.UNKNOWN

        if no_dialog:
            config.runtime.config_type = ConfigType.LOCAL

        if repo_url:
            config.runtime.repo_url = repo_url
            config.runtime.repo_type = RepoType.CLIENT

        return config

    @staticmethod
    def __save_config_file(data: Dict, file_path: Path):
        with open(file_path, 'w') as configfile:
            json.dump(data, configfile, indent=1)

    def save_local_config(self, tmp_config: ConfigEntity):

        data = self._read_full_local_config()
        data.update(tmp_config.model_dump(exclude_none=True, by_alias=True, exclude={'runtime', 'RUNTIME', 'daemon', 'DAEMON'}))

        self.__save_config_file(data=data, file_path=self._local_config_file)

    def save_global_config(self, tmp_config: ConfigEntity):
        data = self._read_full_global_config()
        data.update(tmp_config.model_dump(exclude_none=True, by_alias=True, exclude={'runtime', 'RUNTIME', 'daemon', 'DAEMON'}))
        self.__save_config_file(data=data, file_path=self._global_config_file)

    def save_token_to_config(self, config_type: ConfigType, token: str):
        tmp_config = ConfigEntity()
        tmp_config.main = MainConfigEntity(
            thestage_auth_token=token,
        )

        self.save_full_config(config=tmp_config, config_type=config_type)

    def save_full_config(self, config: ConfigEntity, config_type: ConfigType):

        if config_type == ConfigType.LOCAL:
            self.save_local_config(config)
        elif config_type == ConfigType.GLOBAL:
            self.save_global_config(config)

    def remove_all_config(self,):
        self.remove_global_config()
        self.remove_local_config()
        self.remove_config_env()

    def remove_global_config(self,):
        if self._global_config_path and self._global_config_path.exists():
            self._file_system_service.remove_folder(str(self._global_config_path))

    def remove_local_config(self,):
        if self._local_config_path and self._local_config_path.exists():
            self._file_system_service.remove_folder(str(self._local_config_path))

    @staticmethod
    def remove_config_env():
        os.unsetenv('THESTAGE_CONFIG_DIR')
        os.unsetenv('THESTAGE_CONFIG_FILE')
        os.unsetenv('THESTAGE_CLI_ENV')
        os.unsetenv('THESTAGE_API_URL')
        os.unsetenv('THESTAGE_LOG_FILE')
        os.unsetenv('THESTAGE_AUTH_TOKEN')

    @staticmethod
    def check_for_daemon(config: ConfigEntity,):
        daemon_env_path = Path(THESTAGE_DAEMON_ENV_PATH)
        if daemon_env_path.exists():
            daemon_config = dotenv.dotenv_values(daemon_env_path)
            token_path = daemon_config.get('JWT_PATH')
            backend_api_url = daemon_config.get('BACKEND_URL')
            if daemon_config and token_path:
                daemon_token_path = Path(token_path)
                if daemon_token_path.exists():
                    with open(token_path, 'r') as f:
                        daemon_token = f.readline()
                    if daemon_token:
                        config.daemon.daemon_token = daemon_token.strip().replace('\\n', '')
                        config.daemon.backend_api_url = backend_api_url.strip().replace('\\n', '') if backend_api_url else None
                        config.start_on_daemon = True




class BaseAbstractException(Exception):
    def __init__(self, message: str):
        self._message = message

    def __str__(self):
        return self._message

    def __repr__(self):
        return self._message

    def get_message(self) -> str:
        return self._message



from enum import Enum
from typing import List


class SelfHostedStatusEnumDto(str, Enum):
    AWAITING_CONFIGURATION: str = 'AWAITING_CONFIGURATION'
    RUNNING: str = 'RUNNING'
    TERMINATED: str = 'TERMINATED'
    DELETED: str = 'DELETED'
    UNKNOWN: str = 'UNKNOWN'
    ALL: str = 'ALL'

    @staticmethod
    def find_special_status(statuses: List['SelfHostedStatusEnumDto']) -> bool:
        q = list(filter(lambda x: True if x == SelfHostedStatusEnumDto.ALL else False, statuses))
        return True if q else False




from enum import Enum


class CurrencyTypeEnumDto(str, Enum):
    ETHER: str = 'ETHER'
    STAGI: str = 'STAGI'
    USD: str = 'USD'
    EUR: str = 'EUR'
    USDT: str = 'USDT'
    UNKNOWN: str = 'UNKNOWN'



from enum import Enum
from typing import List


class RentedStatusEnumDto(str, Enum):
    NEW: str = 'NEW'
    AWAITING_RENT: str = 'AWAITING_RENT'
    RENTING: str = 'RENTING'
    RENTED: str = 'RENTED'
    RENTING_FAILED: str = 'RENTING_FAILED'
    AWAITING_TERMINATE: str = 'AWAITING_TERMINATE'
    TERMINATING: str = 'TERMINATING'
    TERMINATED: str = 'TERMINATED'
    TERMINATING_FAILED: str = 'TERMINATING_FAILED'
    UNKNOWN: str = 'UNKNOWN'
    ALL: str = 'ALL'

    @staticmethod
    def find_special_status(statuses: List['RentedStatusEnumDto']) -> bool:
        q = list(filter(lambda x: True if x == RentedStatusEnumDto.ALL else False, statuses))
        return True if q else False



from typing import Dict, Optional

from thestage_core.services.config_provider.config_provider import ConfigProviderCore
from thestage_core.services.clients.thestage_api.api_client import TheStageApiClientCore
from thestage_core.entities.config_entity import ConfigEntity, MainConfigEntity


class ValidationServiceCore:
    _thestage_api_client: TheStageApiClientCore = None

    def __init__(
            self,
            thestage_api_client: TheStageApiClientCore,
            config_provider: ConfigProviderCore,
    ):
        self._thestage_api_client = thestage_api_client
        self._config_provider = config_provider

    @staticmethod
    def is_present_token(config: ConfigEntity) -> bool:
        present_token = True
        if not config:
            present_token = False
        else:
            if not config.main.auth_token:
                present_token = False

            if config.start_on_daemon:
                if config.daemon and config.daemon.daemon_token:
                    present_token = True
                else:
                    present_token = False

        return present_token

    def validate_token(self, new_token: str,) -> bool:
        is_valid: bool = False
        if new_token:
            is_valid = self._thestage_api_client.validate_token(token=new_token)
        return is_valid



import os
from pathlib import Path
from typing import Optional, List

from pydantic import BaseModel, ConfigDict, Field


class FileItemEntity(BaseModel):
    model_config = ConfigDict(use_enum_values=True)

    name: Optional[str] = Field(None)
    path: Optional[str] = Field(None)
    is_file: Optional[bool] = Field(False)
    is_folder: Optional[bool] = Field(False)
    file_size: Optional[int] = Field(None)
    children: List['FileItemEntity'] = Field(default=[])

    @staticmethod
    def build_from_path(path: Path) -> 'FileItemEntity':
        file_stat = os.stat(path)
        return FileItemEntity(
                name=path.name,
                path=str(path.absolute()),
                is_file=path.is_file(),
                is_folder=path.is_dir(),
                file_size=file_stat.st_size,
            )



import os

THESTAGE_CONFIG_DIR = os.getenv('THESTAGE_CONFIG_DIR', '.thestage')
THESTAGE_CONFIG_FILE = os.getenv('THESTAGE_CONFIG_FILE', 'config.json')
THESTAGE_AUTH_TOKEN = os.getenv('THESTAGE_AUTH_TOKEN', None)
THESTAGE_API_URL = os.getenv('THESTAGE_API_URL', 'https://backend.thestage.ai')
THESTAGE_DAEMON_ENV_PATH = os.getenv('THESTAGE_DAEMON_ENV_PATH', '/etc/thestage-daemon-environment')
THESTAGE_DAEMON_TOKEN_PATH = os.getenv('THESTAGE_DAEMON_TOKEN_PATH', '/etc/thestage-daemon-token')
